"use strict";
var DatabaseModule_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseModule = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const typeorm_1 = require("@nestjs/typeorm");
const child_process_1 = require("child_process");
const typeorm_2 = require("typeorm");
const database_config_1 = require("./database.config");
const database_service_1 = require("./service/database.service");
const migration_service_1 = require("./service/migration.service");
const provisioner_service_1 = require("./service/provisioner.service");
const psql_service_1 = require("./service/psql.service");
const database_connection_inject_token_1 = require("./token/database-connection-inject-token");
const spawn_sync_inject_token_1 = require("./token/spawn-sync-inject-token");
let DatabaseModule = DatabaseModule_1 = class DatabaseModule {
    static forRoot(entities, migrations) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const databaseConfig = new database_config_1.DatabaseConfig(entities, migrations);
            const psqlService = new psql_service_1.PsqlService(databaseConfig, child_process_1.spawnSync);
            const databaseService = new database_service_1.DatabaseService(databaseConfig, psqlService);
            const provisionerService = new provisioner_service_1.ProvisionerService(psqlService, databaseConfig);
            const dbCreated = databaseService.doesDbExist();
            if (!dbCreated) {
                databaseService.createDb();
            }
            const providers = [
                { provide: spawn_sync_inject_token_1.spawnSyncInjectToken, useValue: child_process_1.spawnSync },
                { provide: database_config_1.DatabaseConfig, useValue: databaseConfig },
                database_service_1.DatabaseService,
                migration_service_1.MigrationService,
                { provide: provisioner_service_1.ProvisionerService, useValue: provisionerService },
                { provide: psql_service_1.PsqlService, useValue: psqlService },
                { provide: database_connection_inject_token_1.DatabaseConnectionInjectToken, useFactory: () => tslib_1.__awaiter(this, void 0, void 0, function* () { return yield typeorm_2.createConnection(databaseConfig); }) }
            ];
            return {
                imports: [typeorm_1.TypeOrmModule.forRoot(databaseConfig)],
                module: DatabaseModule_1,
                providers,
                exports: providers
            };
        });
    }
};
DatabaseModule = DatabaseModule_1 = tslib_1.__decorate([
    common_1.Module({})
], DatabaseModule);
exports.DatabaseModule = DatabaseModule;
//# sourceMappingURL=database.module.js.map