"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DatabaseService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const child_process_1 = require("child_process");
const path_1 = require("path");
const database_config_1 = require("../database.config");
const psql_service_1 = require("./psql.service");
const LATEST = 'latest';
let DatabaseService = class DatabaseService {
    constructor(config, psql) {
        this.config = config;
        this.psql = psql;
        this.metaTable = 'asa_metadata';
        this.migrationDirectory = 'libs/server/database/src/migration';
    }
    createDb() {
        return this.psql.exec(`CREATE DATABASE ${this.config.database}`);
    }
    createMetaTable() {
        return this.psql.exec(`CREATE TABLE ${this.metaTable} (id varchar(10) NOT NULL, version varchar(40) NOT NULL)`);
    }
    doesDbExist() {
        const count = this.psql.exec(`select count(*) from pg_catalog.pg_database where datname='${this.config.database}'`);
        return Number(count) > 0;
    }
    getVersion(artifactId) {
        return this.psql.exec(`SELECT version FROM ${this.metaTable} WHERE id='${artifactId}'`);
    }
    migrate(requestedVersion = LATEST) {
        if (requestedVersion === LATEST) {
            return child_process_1.execSync('typeorm migration:run', { cwd: path_1.resolve(this.migrationDirectory) }).toString();
        }
        return `migrating to a specific version (like ${requestedVersion}) has not been implemented yet`;
    }
    setVersion(artifactId, artifactVersion) {
        return this.psql.exec([
            `DELETE FROM ${this.metaTable} WHERE id='${artifactId}'`,
            `INSERT INTO ${this.metaTable} (id, version) VALUES ('${artifactId}', '${artifactVersion}')`
        ].join(';\n'));
    }
};
DatabaseService = tslib_1.__decorate([
    common_1.Injectable(),
    tslib_1.__metadata("design:paramtypes", [database_config_1.DatabaseConfig, psql_service_1.PsqlService])
], DatabaseService);
exports.DatabaseService = DatabaseService;
//# sourceMappingURL=database.service.js.map