"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MigrationService = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@nestjs/common");
const fs_1 = require("fs");
const path_1 = require("path");
const typeorm_1 = require("typeorm");
const database_config_1 = require("../database.config");
const database_connection_inject_token_1 = require("../token/database-connection-inject-token");
let MigrationService = class MigrationService {
    constructor(connection, databaseConfig) {
        this.connection = connection;
        this.databaseConfig = databaseConfig;
    }
    onModuleInit() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const version = new Date().getTime();
            yield this.buildSchema(version);
            yield this.applySchemaToDatabase(version);
        });
    }
    applySchemaToDatabase(_version) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            yield this.connection.runMigrations({ transaction: 'all' });
        });
    }
    buildSchema(version) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const { upSqls, downSqls } = yield this.buildSchemaRaw();
            return this.writeSchemaToDisk(version, upSqls, downSqls);
        });
    }
    writeSchemaToDisk(version, upSqls, downSqls) {
        const outDirectory = this.databaseConfig.cli.migrationsDir;
        const name = `V${version}`;
        const filename = name + '.ts';
        const fileContent = this.getTemplate(name, upSqls, downSqls.reverse());
        const path = path_1.resolve(outDirectory, filename);
        fs_1.writeFileSync(path, fileContent, 'utf8');
    }
    buildSchemaRaw() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const sqlInMemory = yield this.connection.driver.createSchemaBuilder().log();
            const upSqls = [];
            const downSqls = [];
            sqlInMemory.upQueries.forEach(upQuery => {
                upSqls.push('        await queryRunner.query(`' +
                    upQuery.query.replace(new RegExp('`', 'g'), '\\`') +
                    '`, ' +
                    JSON.stringify(upQuery.parameters) +
                    ');');
            });
            sqlInMemory.downQueries.forEach(downQuery => {
                downSqls.push('        await queryRunner.query(`' +
                    downQuery.query.replace(new RegExp('`', 'g'), '\\`') +
                    '`, ' +
                    JSON.stringify(downQuery.parameters) +
                    ');');
            });
            return { upSqls, downSqls };
        });
    }
    getTemplate(migrationName, upSqls, downSqls) {
        return `import {MigrationInterface, QueryRunner} from "typeorm";

export class ${migrationName} implements MigrationInterface {
    name = '${migrationName}'

    public async up(queryRunner: QueryRunner): Promise<void> {
${upSqls.join('\n')}
    }

    public async down(queryRunner: QueryRunner): Promise<void> {
${downSqls.join('\n')}
    }
}\n`;
    }
};
MigrationService = tslib_1.__decorate([
    common_1.Injectable(),
    tslib_1.__param(0, common_1.Inject(database_connection_inject_token_1.DatabaseConnectionInjectToken)),
    tslib_1.__metadata("design:paramtypes", [typeorm_1.Connection,
        database_config_1.DatabaseConfig])
], MigrationService);
exports.MigrationService = MigrationService;
//# sourceMappingURL=migration.service.js.map